@@ -20,12 +20,6 @@
 #define LABEL_DATA "gnome-keybinding-properties-label"
 #define MAX_ELEMENTS_BEFORE_SCROLLING 10
 
-#if defined(__powerpc__) && defined (__linux__)
-#define USE_FBLEVEL
-#else
-#undef USE_FBLEVEL
-#endif
-
 typedef enum {
   ALWAYS_VISIBLE,
   N_WORKSPACES_GT
@@ -53,10 +47,6 @@ static const KeyListEntry desktop_key_list[] =
   { "/apps/metacity/global_keybindings/run_command_screenshot", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_window_screenshot", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_terminal", ALWAYS_VISIBLE, 0 },
-#ifdef USE_FBLEVEL
-  { "/apps/gnome_settings_daemon/keybindings/brightness_down", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/brightness_up", ALWAYS_VISIBLE, 0 },
-#endif
   { NULL }
 };
 static const KeyListEntry sounds_key_list[] =
@@ -656,13 +646,15 @@ accel_edited_callback (GtkCellRendererText   *cell,
     {
       if ((tmp_key.keyval >= GDK_a && tmp_key.keyval <= GDK_z)
 	   || (tmp_key.keyval >= GDK_A && tmp_key.keyval <= GDK_Z)
-	   || (tmp_key.keyval >= GDK_kana_fullstop && tmp_key.keyval <= GDK_kana_switch)
-	   || (tmp_key.keyval >= GDK_Arabic_comma && tmp_key.keyval <= GDK_Arabic_switch)
+	   || (tmp_key.keyval >= GDK_kana_fullstop && tmp_key.keyval <= GDK_semivoicedsound)
+	   || (tmp_key.keyval >= GDK_Arabic_comma && tmp_key.keyval <= GDK_Arabic_sukun)
 	   || (tmp_key.keyval >= GDK_Serbian_dje && tmp_key.keyval <= GDK_Cyrillic_HARDSIGN)
-	   || (tmp_key.keyval >= GDK_Greek_ALPHAaccent && tmp_key.keyval <= GDK_Greek_switch)
-	   || (tmp_key.keyval >= GDK_hebrew_doublelowline && tmp_key.keyval <= GDK_Hebrew_switch)
+	   || (tmp_key.keyval >= GDK_Greek_ALPHAaccent && tmp_key.keyval <= GDK_Greek_omega)
+	   || (tmp_key.keyval >= GDK_hebrew_doublelowline && tmp_key.keyval <= GDK_hebrew_taf)
 	   || (tmp_key.keyval >= GDK_Thai_kokai && tmp_key.keyval <= GDK_Thai_lekkao)
-	   || (tmp_key.keyval >= GDK_Hangul && tmp_key.keyval <= GDK_Hangul_J_YeorinHieuh)) {
+	   || (tmp_key.keyval >= GDK_Hangul && tmp_key.keyval <= GDK_Hangul_Special)
+	   || (tmp_key.keyval >= GDK_Hangul_Kiyeog && tmp_key.keyval <= GDK_Hangul_J_YeorinHieuh)
+	   || tmp_key.keyval == GDK_Mode_switch) {
         GtkWidget *dialog;
 	char *name;
 
